/*
 * Copyright (c) 2022 NVIDIA CORPORATION & AFFILIATES, ALL RIGHTS RESERVED.
 *
 * This software product is a proprietary product of NVIDIA CORPORATION &
 * AFFILIATES (the "Company") and all right, title, and interest in and to the
 * software product, including all associated intellectual property rights, are
 * and shall remain exclusively with the Company.
 *
 * This software product is governed by the End User License Agreement
 * provided with the software product.
 *
 */

#ifndef DOCA_SHA_ID_GENERATOR_H_
#define DOCA_SHA_ID_GENERATOR_H_

#include <stdint.h>
#include <stddef.h>
#include <stdlib.h>
#include <doca_error.h>

#ifdef __cplusplus
extern "C" {
#endif

#define DOCA_SHA_ID_GENERATOR_INVALID_ID 0

struct doca_sha_id_generator;

/* Creates an ID allocator that allocates IDs in the range [1, max_id]. */
struct doca_sha_id_generator *doca_sha_id_generator_create(uint32_t max_id);

void doca_sha_id_generator_destroy(struct doca_sha_id_generator *id_generator);

/* Returns the smallest free ID in the range [1, max_id]. */
uint32_t doca_sha_id_generator_get(struct doca_sha_id_generator *id_generator);

/* Frees a previously allocated ID, the next may reuse the same ID. */
doca_error_t doca_sha_id_generator_put(struct doca_sha_id_generator *id_generator, uint32_t id);

#ifdef __cplusplus
}
#endif

#endif /* DOCA_SHA_ID_GENERATOR_H_ */
