/*
 * Copyright (c) 2022 NVIDIA CORPORATION & AFFILIATES, ALL RIGHTS RESERVED.
 *
 * This software product is a proprietary product of NVIDIA CORPORATION &
 * AFFILIATES (the "Company") and all right, title, and interest in and to the
 * software product, including all associated intellectual property rights, are
 * and shall remain exclusively with the Company.
 *
 * This software product is governed by the End User License Agreement
 * provided with the software product.
 *
 */

#ifndef DOCA_SHA_OFFLOAD_LIB_H_
#define DOCA_SHA_OFFLOAD_LIB_H_

#include <stdbool.h>
#include <stdio.h>
#include <stdlib.h>
#include <stddef.h>
#include <unistd.h>

#define DOCA_SHA1_CBLOCK       64
#define DOCA_SHA1_DIGEST_LEN   20
#define DOCA_SHA256_CBLOCK     64
#define DOCA_SHA256_DIGEST_LEN 32
#define DOCA_SHA512_CBLOCK     128
#define DOCA_SHA512_DIGEST_LEN 64

/**
 * A forward declaration of the data structure used for maintaining doca_sha engine state.
 */
struct doca_sha_engine_state;

/**
 * A forward declaration of the data structure used for doca_sha's offload context.
 */
struct doca_sha_ctx;

/**
 * enum for doca_sha supported sha function types.
 */
enum doca_sha_func_type {
	DOCA_SHA_FUNC_TYPE_SHA1 = 0,
	DOCA_SHA_FUNC_TYPE_SHA256,
	DOCA_SHA_FUNC_TYPE_SHA512,
};

/**
 * Functions to init/destroy the doca_sha_engine,
 * and reset the doca_sha_engine_state.
 */
int doca_sha_engine_create(struct doca_sha_engine_state **state, int debug_level);
int doca_sha_engine_init(struct doca_sha_engine_state *state, const char *pci_addr, int debug_level);
int doca_sha_engine_finish(struct doca_sha_engine_state *state, int debug_level);
int doca_sha_engine_destroy(struct doca_sha_engine_state *state, int debug_level);
int doca_sha_reset_engine_state(struct doca_sha_engine_state *state, int debug_level);

/**
 * Function callbacks to EVP_Digest_init/update/update/copy.
 */
int doca_sha_init_v1(struct doca_sha_ctx *ctx, enum doca_sha_func_type, struct doca_sha_engine_state *state,
			int debug_level);
int doca_sha_update_v1(struct doca_sha_ctx *ctx, unsigned char *msg, size_t msg_len, int debug_level);
int doca_sha_final_v1(unsigned char *digest, struct doca_sha_ctx *ctx, int debug_level);
int doca_sha_copy_v1(struct doca_sha_ctx *to, const struct doca_sha_ctx *from, int debug_level);
size_t get_size_of_doca_sha_ctx(void);

#endif /* DOCA_SHA_OFFLOAD_LIB_H_ */
